@@ -1,13 +1,13 @@
 from typing import List, Callable, Optional, Dict, Union
 from pandas import Series, DataFrame
 import numpy as np
+import pandas as pd
 from tqdm import tqdm
 from joblib import Parallel, delayed
 
 from pymsm.competing_risks_model import CompetingRisksModel
 from pymsm.event_specific_fitter import CoxWrapper, EventSpecificFitter
-
-np.seterr(divide="ignore", invalid="ignore")  # TODO
+from pymsm.state_diagram import state_diagram
 
 
 def default_update_covariates_function(
@@ -79,6 +79,7 @@ class MultiStateModel:
         covariate_names (List[str], optional): Optional list of covariate names to be used. Defaults to None.
         event_specific_fitter (EventSpecificFitter, optional): This class holds the model that will be fitter inside the CompetingRisksModel. Defaults to CoxWrapper.
         competing_risk_data_format (bool, optional): A boolean indicating the format of the dataset parmeter, if False - the dataset is assumed to be a list of PathObjects, if True - the dataset is assumed to be a dataframe which is compatible in format for fitting the CompetingRiskModel class. Defaults to False.
+        states_labels (Dict[int, str], optional): A dictionary of short state labels. Defaults to None.
 
     Attributes:
         state_specific_models (Dict[int, CompetingRisksModel]): A dictionary of CompetingRisksModel objects, one for each state. Available after running the "fit" function.
@@ -98,6 +99,7 @@ class MultiStateModel:
         covariate_names: List[str] = None,
         event_specific_fitter: EventSpecificFitter = CoxWrapper,
         competing_risk_data_format: bool = False,
+        state_labels: Dict[int, str] = None,
     ):
         self.dataset = dataset
         self.terminal_states = terminal_states
@@ -109,8 +111,12 @@ class MultiStateModel:
         self._samples_have_weights: bool = False
         self._competing_risk_data_format = competing_risk_data_format
         self._event_specific_fitter = event_specific_fitter
+        self.state_labels = state_labels
+        self.transition_table: DataFrame = None
 
-        if not self._competing_risk_data_format:
+        if self._competing_risk_data_format:
+            self.competing_risk_dataset = dataset
+        else:
             self._assert_valid_input()
 
     def fit(self, verbose: int = 1) -> None:
@@ -133,10 +139,12 @@ class MultiStateModel:
 
             model = self._fit_state_specific_model(state, verbose)
             self.state_specific_models[state] = model
+        if verbose >= 1:
+            self.plot_state_diagram()
 
     def _assert_valid_input(self) -> None:
         """Checks that the dataset is valid for running the multi state competing risk model"""
-        # Check the number os time is either equal or one less than the number of states
+        # Check the number of times is either equal or one less than the number of states
         for obj in self.dataset:
             n_states = len(obj.states)
             n_times = len(obj.time_at_each_state)
@@ -221,11 +229,10 @@ class MultiStateModel:
                 transition_row["time_entry_to_origin"] = time_entry_to_origin
                 transition_row["time_transition_to_target"] = time_transition_to_target
                 transition_row.update(covs_entering_origin.to_dict())
-                self.competing_risk_dataset = self.competing_risk_dataset.append(
-                    transition_row, ignore_index=True
+                self.competing_risk_dataset = pd.concat(
+                    [self.competing_risk_dataset, pd.DataFrame([transition_row])],
+                    ignore_index=True,
                 )
-                # TODO change to concat due to:
-                # FutureWarning: The frame.append method is deprecated and will be removed from pandas in a future version. Use pandas.concat instead.
 
                 if (
                     target_state == RIGHT_CENSORING
@@ -249,8 +256,58 @@ class MultiStateModel:
         self.competing_risk_dataset["target_state"] = self.competing_risk_dataset[
             "target_state"
         ].astype(int)
+
+        # Create default state_labels if None provided
+        if self.state_labels is None:
+            unique_states = pd.unique(
+                self.competing_risk_dataset[
+                    ["origin_state", "target_state"]
+                ].values.ravel("K")
+            )  # find unique possible states
+            unique_states.sort()  # sort
+            unique_states = unique_states[unique_states > 0]  # drop censored
+            self.state_labels = dict(
+                zip(unique_states, [str(s) for s in unique_states])
+            )
+
         return self.competing_risk_dataset
 
+    def prep_transition_table(self):
+        """This function creates a transition matrix from the dataset"""
+        if self.competing_risk_dataset is None:
+            self._prepare_dataset_for_competing_risks_fit()
+        # Create transition matrix
+        self.transition_matrix = pd.crosstab(
+            self.competing_risk_dataset["origin_state"],
+            self.competing_risk_dataset["target_state"],
+        )
+        # Rename rows and columns and get a transition table
+        self.transition_table = self.transition_matrix.copy()
+        rename_dict = self.state_labels.copy()
+        rename_dict[0] = "Censored"
+        self.transition_table.rename(columns=rename_dict, inplace=True)
+        self.transition_table.rename(index=rename_dict, inplace=True)
+        return self.transition_table
+
+    def plot_state_diagram(self):
+        if self.transition_table is None:
+            self.prep_transition_table()
+        graph = """\n"""
+        for s, state_label in self.state_labels.items():
+            graph += f"""s{s} : ({s}) {state_label}\n"""
+        for origin_state, row in self.transition_matrix.iterrows():
+            for target_state in row.index:
+                if target_state == 0:  # Censored transition
+                    continue
+                if row[target_state] == 0:  # Empty transition
+                    continue
+                num_transitions = row[target_state]
+                graph += (
+                    f"""s{origin_state} --> s{target_state}: {num_transitions} \n"""
+                )
+        graph += """\n"""
+        return state_diagram(graph)
+
     def _fit_state_specific_model(
         self, state: int, verbose: int = 1
     ) -> CompetingRisksModel:
@@ -298,8 +355,6 @@ class MultiStateModel:
         assert n_random_samples > 0
         assert isinstance(max_transitions, int)
         assert max_transitions > 0
-        assert isinstance(n_jobs, int)
-        assert n_jobs >= -1
         assert isinstance(print_paths, bool)
 
     def run_monte_carlo_simulation(
@@ -496,9 +551,12 @@ class MultiStateModel:
         )
 
         probability_for_each_t = np.nancumsum(hazard * survival)
-        probability_for_each_t_given_next_state = (
-            probability_for_each_t / probability_for_each_t.max()
-        )  # TODO this raises warnings and we should create better error handling
+        try:
+            probability_for_each_t_given_next_state = (
+                probability_for_each_t / probability_for_each_t.max()
+            )
+        except:
+            pass  # TODO this raises warnings and we should create better error handling
 
         # take the first event time whose probability is less than or equal to eps
         # if we drew a very small eps, use the minimum observed time
